@@ -3,7 +3,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %% 
-%%     $Id: sdl_util.erl,v 1.1 2004/03/30 07:46:20 bjorng Exp $
+%%     $Id: sdl_util.erl,v 1.2 2005/06/29 14:45:39 dgud Exp $
 %%
 %%%----------------------------------------------------------------------
 %%% File    : sdl_util.erl
@@ -318,7 +318,7 @@ copySdlImage2GLArray(Image, Mem, Bpp) when record(Image, sdl_surface) ->
 copySdlImage2GLArray({surfacep, Image}, #sdlmem{bin=Mem}, Bpp) 
   when Bpp == 3; Bpp == 4 ->
     sdl:send_bin(Mem),
-    <<1:8>> = call(?SDL_UTIL_copySdlImage2GLArray, <<Image:32,Bpp:8>>),
+    <<1:8>> = call(?SDL_UTIL_copySdlImage2GLArray, <<Image:?_PTR,Bpp:8>>),
     ok.
 
 mem_size(?GL_BYTE, Size) -> Size;
